import os
import sys

from lapa_commons.main import read_configuration_from_file_path
from square_logger.main import SquareLogger

try:
    config_file_path = (
        os.path.dirname(os.path.abspath(__file__))
        + os.sep
        + "data"
        + os.sep
        + "config.ini"
    )
    ldict_configuration = read_configuration_from_file_path(config_file_path)

    # get all vars and typecast
    config_str_host_ip = ldict_configuration["ENVIRONMENT"]["HOST_IP"]
    config_int_host_port = int(ldict_configuration["ENVIRONMENT"]["HOST_PORT"])
    config_str_log_file_name = ldict_configuration["ENVIRONMENT"]["LOG_FILE_NAME"]

    # Initialize logger
    global_object_square_logger = SquareLogger(config_str_log_file_name)
except Exception as e:
    print(
        "\033[91mMissing or incorrect config.ini file.\n"
        "Error details: " + str(e) + "\033[0m"
    )
    sys.exit()



import base64
from http import HTTPStatus

import bcrypt
from fastapi import FastAPI, status
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import JSONResponse
from lapa_database_helper.main import LAPADatabaseHelper
from lapa_database_structure.main import DatabasesEnum, SchemaEnum, TablesEnum
from uvicorn import run

from lapa_authentication.configuration import (
    config_int_host_port,
    config_str_host_ip,
    global_object_square_logger,
)
from lapa_authentication.entity.Models import RegisterUser
from lapa_authentication.utils.CommonEnums import (
    HashingAlgorithm,
    User,
    UserRegistration,
    UserValidation,
)
from lapa_authentication.utils.Helper import (
    get_hash_algorithm_id,
    get_user_registration_id,
    get_user_validation_status_id,
)

local_object_lapa_database_helper = LAPADatabaseHelper()

app = FastAPI()

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_methods=["*"],
    allow_headers=["*"],
)


@app.get("/")
@global_object_square_logger.async_auto_logger
async def root():
    return JSONResponse(
        status_code=status.HTTP_200_OK, content={"text": "lapa_authentication"}
    )


@app.post("/register")
@global_object_square_logger.async_auto_logger
async def register(register_user: RegisterUser):
    """
    Description - This API endpoint is used to register user into the authentication database.
    """
    ldict_user_data = dict()
    try:
        lbool_user_created = False

        # =========================================================
        # Check whether the user already exists in the user table
        # =========================================================
        _register_user = register_user.model_dump()
        llst_user_found = local_object_lapa_database_helper.get_rows(
            database_name=DatabasesEnum.authentication.value,
            schema_name=SchemaEnum.public.value,
            table_name=TablesEnum.user.value,
            filters={User.user_email_id.value: _register_user["email"]},
            ignore_filters_and_get_all=False,
        )

        # =========================================================
        # If Yes --> Return msg saying user already exists with the same email id
        # =========================================================
        if len(llst_user_found) > 0:
            return JSONResponse(
                status_code=HTTPStatus.CONFLICT,
                content={
                    "user_created": lbool_user_created,
                    "message": "User already exists with the same email id",
                },
            )
        else:
            # =========================================================
            # If No --> Create user in the user table
            # Generate a random salt
            # =========================================================
            lbyte_salt = bcrypt.gensalt()

            # =========================================================
            # Hash the password with the salt
            # =========================================================
            lbyte_hashed_password = bcrypt.hashpw(
                _register_user["password"].encode("utf-8"), lbyte_salt
            )

            ldict_user_data[User.user_email_id.value] = _register_user["email"]
            ldict_user_data[User.user_password_salt.value] = base64.b64encode(
                lbyte_salt
            ).decode("utf-8")
            ldict_user_data[User.user_password_hash.value] = base64.b64encode(
                lbyte_hashed_password
            ).decode("utf-8")

            # =========================================================
            # Fetch user_validation_status_id where status_description = 'pending'
            # =========================================================
            ldict_user_data[
                UserValidation.user_validation_status_id.value
            ] = get_user_validation_status_id()

            # =========================================================
            # Fetch user_registration_id where registration_description = _register_user['registration_type']
            # =========================================================
            ldict_user_data[
                UserRegistration.user_registration_id.value
            ] = get_user_registration_id(_register_user["registration_type"])
            ldict_user_data[
                HashingAlgorithm.hash_algorithm_id.value
            ] = get_hash_algorithm_id()
            insert_row_response = local_object_lapa_database_helper.insert_rows(
                database_name=DatabasesEnum.authentication.value,
                schema_name=SchemaEnum.public.value,
                table_name=TablesEnum.user.value,
                data=[ldict_user_data],
            )
            if len(insert_row_response) == 1:
                # =========================================================
                # Value inserted into database successful
                # =========================================================
                if "user_id" in insert_row_response[0]:
                    lbool_user_created = True
                    return JSONResponse(
                        status_code=HTTPStatus.CREATED,
                        content={
                            "user_created": lbool_user_created,
                            "message": "User created successfully",
                        },
                    )
            else:
                return JSONResponse(
                    status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                    content={
                        "user_created": lbool_user_created,
                        "message": "User not created",
                    },
                )
    except Exception:
        raise
    finally:
        del ldict_user_data


if __name__ == "__main__":
    try:
        run(app, host=config_str_host_ip, port=config_int_host_port)

    except Exception as exc:
        global_object_square_logger.logger.critical(exc, exc_info=True)



import re

from pydantic import BaseModel, constr, field_validator, EmailStr


class RegisterUser(BaseModel):
    email: EmailStr
    password: constr(min_length=8, max_length=50)
    registration_type: str

    @field_validator("email")
    def validate_email(cls, email: str):
        try:
            # ================================================================
            # Basic email format validation
            # ================================================================
            if "@" not in email:
                raise ValueError("Invalid email format")

            # ================================================================
            # We can add more complex validation for the domain if needed
            # For simplicity, we're just checking for the presence of a dot (.)
            # ================================================================
            if "." not in email.split("@")[1]:
                raise ValueError("Invalid email domain")

            return email
        except Exception:
            raise

    @field_validator("password")
    def validate_password(cls, password: str):
        try:
            # ================================================================
            # Check for at least one special character, one digit, and one letter
            # ================================================================
            if not re.search(r"(?=.*[!@#$%^&*(),.?\":{}|<>])(?=.*\d)(?=.*[a-zA-Z])", password):
                raise ValueError("Password must contain at least one special character, one digit, and one letter")

            return password
        except Exception:
            raise

    @field_validator("registration_type")
    def validate_registration_type(cls, registration_type):
        try:
            allowed_types = ["email"]
            if registration_type.lower() not in allowed_types:
                raise ValueError(f"Invalid registration type. Allowed types: {', '.join(allowed_types)}")
            return registration_type.lower()
        except Exception:
            raise



from lapa_database_helper.main import LAPADatabaseHelper
from lapa_database_structure.main import DatabasesEnum, SchemaEnum, TablesEnum

from lapa_authentication.utils.CommonEnums import (
    HashingAlgorithm,
    UserRegistration,
    UserValidation,
)

local_object_lapa_database_helper = LAPADatabaseHelper()


def get_user_validation_status_id(pstr_status_description: str = "pending") -> int:
    """
    Description - This function is used to retrieve the status_id for the pending status when the user is created
    for the first time in the authentication system.
    :param pstr_status_description: pending
    :return: status_id
    """
    ldict_filter_condition = dict()
    try:
        ldict_filter_condition[
            UserValidation.status_description.value
        ] = pstr_status_description
        user_validation = local_object_lapa_database_helper.get_rows(
            database_name=DatabasesEnum.authentication.value,
            schema_name=SchemaEnum.public.value,
            table_name=TablesEnum.user_validation_status.value,
            filters=ldict_filter_condition,
            ignore_filters_and_get_all=False,
        )
        return user_validation[0][UserValidation.user_validation_status_id.value]
    except Exception:
        raise
    finally:
        del ldict_filter_condition


def get_user_registration_id(pstr_registration_description: str) -> int:
    """
    Description - This function is used to retrieve the registration_id for the registration_description.
    :param pstr_registration_description: email
    :return: registration_id
    """
    ldict_filter_condition = dict()
    try:
        ldict_filter_condition[
            UserRegistration.registration_description.value
        ] = pstr_registration_description

        user_registration = local_object_lapa_database_helper.get_rows(
            database_name=DatabasesEnum.authentication.value,
            schema_name=SchemaEnum.public.value,
            table_name=TablesEnum.user_registration.value,
            filters=ldict_filter_condition,
            ignore_filters_and_get_all=False,
        )
        return user_registration[0][UserRegistration.user_registration_id.value]
    except Exception:
        raise
    finally:
        del ldict_filter_condition


def get_hash_algorithm_id(pstr_algorithm_name: str = "bcrypt") -> int:
    """
    Description - This function is used to retrieve the hash_algorithm_id for the algorithm_name.
    :param pstr_algorithm_name: bcrypt
    :return: hash_algorithm_id
    """
    ldict_filter_condition = dict()
    try:
        ldict_filter_condition[
            HashingAlgorithm.algorithm_name.value
        ] = pstr_algorithm_name
        hashing_algorithm = local_object_lapa_database_helper.get_rows(
            database_name=DatabasesEnum.authentication.value,
            schema_name=SchemaEnum.public.value,
            table_name=TablesEnum.hashing_algorithm.value,
            filters=ldict_filter_condition,
            ignore_filters_and_get_all=False,
        )
        return hashing_algorithm[0][HashingAlgorithm.hash_algorithm_id.value]
    except Exception:
        raise
    finally:
        del ldict_filter_condition



from enum import Enum


class User(Enum):
    user_email_id = "user_email_id"
    user_password_salt = "user_password_salt"
    user_password_hash = "user_password_hash"


class UserValidation(Enum):
    user_validation_status_id = "user_validation_status_id"
    status_description = "status_description"


class UserRegistration(Enum):
    user_registration_id = "user_registration_id"
    registration_description = "registration_description"


class HashingAlgorithm(Enum):
    hash_algorithm_id = "hash_algorithm_id"
    algorithm_name = "algorithm_name"
